-- Pin by Jadya
-- EU - Well of Eternity
local addonName, addonTable = ...
local L = addonTable.L

local animationspeed = 30

local notes = addonTable.NoteFunctions
local settings = addonTable.settings
local reminder = addonTable.ReminderFunctions

--[[

 - note -
 
 .id				id
 .name				name
 .text				text
 .pin				pin frame
 .fresh				resets the text when focus is gained
 .status			open/opening/closing/closed
 .maxWidth			maxWidth with either horizontal and vertical orientation
 .orientation		horizontal/vertical
 .combat_behavior   0 = normal / 1 = hide / 2 = click through
 
]]

local function getNewID(t)
 local i = 1
 while PinSaves.notes[i] do
  i = i + 1
 end
 return i
end

function notes:New(t)
 local id = getNewID(t)
 local name = addonName.."Note"..id
 local f = CreateFrame("Frame", name, UIParent, "PinNote")
 f.id = id
 f.name = L["new_note"]
 f:SetPoint("LEFT")
 f.status = "open"
 f.fresh = true
 f.text:SetText(L["write_your_text_here"])
 f.orientation = "horizontal"
 f.maxWidth = f:GetWidth()
 f.combat_behavior = 2

 PinSaves.notes[id] = { pin = {}, text = {} }
 notes:Save(f)

 addonTable.notes[id] = f
 return id
end

function notes:Load(id)
 local t = PinSaves.notes[id] 
 local f = CreateFrame("Frame", addonName.."Note"..id, UIParent, "PinNote")
 f:SetSize(1,1)
 f.id              = id
 f.name            = t.name
 f.status          = t.status
 f.orientation     = t.orientation
 f.maxWidth        = t.maxWidth
 f.text:SetText(t.text)
 f.fresh           = t.fresh
 f:SetBackdropColor(unpack(t.backdrop))
 f:SetBackdropBorderColor(unpack(t.backdrop_border))

 local a1, _, a2, x , y = unpack(t.pos)
 f:SetPoint(a1, UIParent, a2, x, y)
 
 -- reminder
 f.reminder_enabled = t.reminder_enabled
 f.reminder_type    = t.reminder_type
 f.reminder_hour    = t.reminder_hour
 f.reminder_minute  = t.reminder_minute
 f.reminder_date     = t.reminder_date
 
 for i = 1,7 do
  f["reminder_weekday"..i] = t["reminder_weekday"..i]
 end
  
 -- pin
 f.pin:SetBackdropColor(unpack(t.pin.backdrop))
 f.pin:SetBackdropBorderColor(unpack(t.pin.backdrop_border))
 
 f.pin:SetWidth(t.pin.w)
 f.pin:SetHeight(t.pin.h)
 
 notes:AnchorPin(f, a1)

 -- text
 f.text:SetFont(unpack(t.font))
 f.text:SetTextColor(unpack(t.font_color))
 
 f.combat_behavior = t.combat_behavior
 
 addonTable.notes[id] = f

 reminder:SetReminder(f, true)
end

function notes:SetAttr(note, k, v)
 note[k] = v
 PinSaves.notes[note.id][k] = v 
end

function notes:Save(note)
 local t = PinSaves.notes[note.id]
 t.status           = note.status
 t.name             = note.name
 t.orientation      = note.orientation
 t.maxWidth         = note.maxWidth
 t.text             = note.text:GetText()
 t.fresh            = note.fresh
 t.combat_behavior  = note.combat_behavior
 t.reminder_enabled = note.reminder_enabled
 t.reminder_type    = note.reminder_type
 t.reminder_hour    = note.reminder_hour
 t.reminder_minute  = note.reminder_minute
 t.reminder_date     = note.reminder_date 
 
 for i = 1,7 do
  t["reminder_weekday"..i] = note["reminder_weekday"..i]
 end
 
 -- font
 local name, height, flags = note.text:GetFont()
 if t.font then
  t.font[1] = name
  t.font[2] = height
  t.font[3] = flags
 else
  t.font       = {name, height, flags}
 end
 
 notes:SavePosition(note)
 notes:SaveColors(note)
end

function notes:SavePosition(note)
 local t = PinSaves.notes[note.id]

 local a1, _, a2, x, y = note:GetPoint()
 t.pos             = {a1, nil, a2, x, y}

 -- pin
 t.pin.w = note.pin:GetWidth()
 t.pin.h = note.pin:GetHeight()
end

function notes:SaveColors(note)
 local t = PinSaves.notes[note.id]
 
 local function save(key, r, g, b, a)
  if t[key] then
   t[key][1] = r
   t[key][2] = g
   t[key][3] = b
   t[key][4] = a
  else
   t[key] = {r,g,b,a}
  end
 end

 save("backdrop", note:GetBackdropColor())
 save("backdrop_border", note:GetBackdropBorderColor())
 save("font_color", note.text:GetTextColor())
 t = t.pin
 save("backdrop", note.pin:GetBackdropColor())
 save("backdrop_border", note.pin:GetBackdropBorderColor())
end

function notes:Delete(note)
 if not note then return end
 
 PinSaves.notes[note.id] = nil
 note:Hide()
end

------------------------------------------------------------------------

local tooltips

local function getTooltip()
 if not tooltips then tooltips = {} end
 
 local result
 
 for _,v in pairs(tooltips) do
  if not v:IsVisible() then
   return v
  end
 end
 
 local i = #tooltips + 1
 result = CreateFrame("GameTooltip", "PinMessageTooltip"..i, UIParent, "GameTooltipTemplate")
 tooltips[i] = result

 return result
end

function notes:ShowMessage(note, s, duration)
 local tt = getTooltip()

 local p = note.pin:GetPoint()
 local side = notes:GetSide(p)

 if side == "LEFT" or side == "BOTTOM" then
  tt:SetOwner(note.pin, "ANCHOR_TOPRIGHT")
 elseif side == "RIGHT" then
  tt:SetOwner(note.pin, "ANCHOR_TOPLEFT")
 else
  tt:SetOwner(note.pin, "ANCHOR_BOTTOMLEFT")
 end
 tt:SetBackdropColor(0,0,0,0.1)
 tt:SetBackdropBorderColor(0,0,0,0)
 tt:ClearLines()
 tt:AddLine(s)
 tt:Show()
 C_Timer.After(duration or 2, function() tt:Hide() end)
end

------------------------------------------------------------------------
function notes:SetMaxWidth(note, w)
 notes:SetAttr(note, "maxWidth", w)
 if note.status == "open" then note:SetWidth(w) end
end

function notes:SetPinWidth(note, w)
 note.pin:SetWidth(w)
 PinSaves.notes[note.id].pin.w = w
end

function notes:SetPinHeight(note, h)
 note.pin:SetHeight(h)
 PinSaves.notes[note.id].pin.h = h
end

function notes:SetFont(note, ...)
 note.text:SetFont(...)
 PinSaves.notes[note.id].font = {...}
end

local function setStatus(note, s)
 if s == "open" then
  note.text:Show()
  PinSaves.notes[note.id].status = s
 elseif s == "closed" then
  PinSaves.notes[note.id].status = s
 elseif s == "closing" then
  note.text:Hide()
 end
 note.status = s
end

local function getNoteSize(note)
 return note.orientation == "vertical" and note:GetHeight() or note:GetWidth()
end

local function setNoteSize(note, v)
 if note.orientation == "vertical" then note:SetHeight(v) else note:SetWidth(v) end
end

local function getMaxSize(note)
 return note.orientation == "vertical" and note.text:GetHeight() or note.maxWidth
end

function notes:Toggle(note, openonly)
 if note.status == "open" or note.status == "opening" then
  if not openonly then
   setStatus(note, "closing")
  else
   return
  end
 elseif note.status == "closed" or note.status == "closing" then
  if note.orientation == "vertical" then 
   note:SetWidth(note.maxWidth)
  else
   Pin_OnTextChanged(note.text)
  end
  setStatus(note, "opening")
 end

 if note.ticker then note.ticker:Cancel() end

 -- ticker
 note.ticker = C_Timer.NewTicker(0.02,
                function()
				 local x = getNoteSize(note)
				 local maxSize = getMaxSize(note)
				 if note.status == "opening" then
				  if x >= maxSize then
				   setNoteSize(note, maxSize)
				   note.ticker:Cancel()
				   setStatus(note, "open")
				   return
				  else
				   setNoteSize(note, x + animationspeed)
				  end
				 elseif note.status == "closing" then
				  if x <= 0 then
				   setNoteSize(note, 1)
				   note.ticker:Cancel()
				   setStatus(note, "closed")
				   return
				  else
				   setNoteSize(note, x - animationspeed)
				  end
				 end
                end)
end


local anchors = { TOPLEFT      = {"RIGHT", "LEFT", "horizontal"},
                  BOTTOMLEFT   = {"RIGHT", "LEFT", "horizontal"},
                  LEFT         = {"RIGHT", "LEFT", "horizontal"},
                  TOP          = {"BOTTOM", "TOP", "vertical"},
				  TOPRIGHT     = {"LEFT", "RIGHT", "horizontal"},
				  BOTTOMRIGHT  = {"LEFT", "RIGHT", "horizontal"},
				  RIGHT        = {"LEFT", "RIGHT", "horizontal"},
				  BOTTOM       = {"TOP", "BOTTOM", "vertical"},
				}

function notes:GetSide(i)
 return anchors[i] and anchors[i][2] or "LEFT"
end
				
function notes:UpdateSize(note)
 local h = note.text:GetHeight()

 if note.status == "closed" then
  note.text:Hide()
  if note.orientation == "vertical" then
   note:SetHeight(1)
   note:SetWidth(note.maxWidth)
  else
   note:SetWidth(1)
   if note:GetHeight() ~= h then
    note:SetHeight(h)
   end
  end
 elseif note.status == "open" then
  note.text:Show()
  if note:GetHeight() ~= h then
   note:SetHeight(h)
  end
  note:SetWidth(note.maxWidth)
 end
end

function notes:AnchorPin(note, side)
 if note.lastside ~= side then
  note.pin:ClearAllPoints()
  note.pin:SetPoint(anchors[side][2], note, anchors[side][1])
  note.orientation = anchors[side][3]
  note.lastside = side
  
  notes:UpdateSize(note)
 end
end


